#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## bonding.pl for bonding in /root
## 
## ######### PROJECT DESCRIPTION : ###
## 
## Creation, demarrage, arret de bonding
## 
## ###################################
## 
## Made by husianycia cedric
## Login   <root@intellique.com>
## 
## Started on  Mon Jun  6 15:29:31 2005 husianycia cedric
## Last update Thu Sep 22 13:03:13 2005 husianycia cedric
## 
## ###################################
## Version

my	$Version = "1.01-7";

## ###################################
## Librairies

use	strict;
use	Objet_Log;		# Objet de log Intellique
use	Objet_Conf;		# Objet de Conf Intellique
use	Objet_Event;		# Objet de gestion d'evenements Intellique
use	Getopt::Long;		# paramtres ligne de commande
use	Time::HiRes qw(time);	# Gestion de l'heure fine

## ###################################
## Declaration de variables globales

		# Variable option
my	$mode		= "";
my	$addfram	= "";
my	$method		= "";
my	$path		= "";
my	$iface		= "";
my	$debug		= "";
my	$help		= "";
my	$ifup		= "";
my	$state		= "";

		# Objet Event
my	$event;
		# Objet Log
my	$log;
		# Objet Conf
my	$conf;
		# Chemin du fichier de configuration
my	$path_conf	= "/etc/storiq/bonding.ini";
		# Chemin du fichier de log
my	$path_log	= "/var/log/storiq/bonding.log.DEFAUT";

## ###################################
## Recuperation des arguments

sub	getCLOptions {
    Getopt::Long::Configure('noignore_case');
      unless (
	      GetOptions(
			 'mode|m=s' => \$mode,
			 'addfram|a=s' => \$addfram,
			 'ifup|I=s' => \$ifup,
			 'debug|d' => \$debug,
			 'method|e=s' => \$method,
			 'path|p=s' => \$path,
			 'iface|i=s' => \$iface,
			 'help|h' => \$help,
			 'state|s' => \$state
			 )
	      )
      {
	  die "abandon : option incorrecte \n";
      }
}

## ###################################
## Print Help

sub	print_help {

    print <<EOF;
	
	Nom du programme	: $0 - Gestion de Bonding
	Version			: $Version
	Description		: Script de gestion du module Bonding.
	
      Options:

	--help | -h		Affiche cette rubrique d'aide
	
	--state | -s	        Renvoi l'etat du Bonding
				(0|1)
	
	--debug | -D		Fonctionnement verbeux

	--mode | -m		Type de demarrage du script :
				(start|stop|restart|force-reload)

	--addfram | -a		Famille de l'adresse de l'interface

	--iface | -i		Nom physique de l'interface
	
	--method | -e		Methode de l'interface (e.g., "static")

	--path | -p		La commande cherche le chemin : 
				/usr/local/sbin
				/usr/local/bin
				/usr/sbin
				/usr/bin
				/sbin
				/bin

	Contact : Cedric HUSIANYCIA ( husiana\@intellique.com )
	Copyright (c) Intellique 2004 
	All rights reserved.
	
EOF
}

## ###################################
## 'Parsing du mode

sub	parse_mode {
    $mode = shift;

    if ($mode eq "force-reload" || $mode eq "reload")
    {
	stoping();
	starting();
    } 
    elsif ($mode eq "start") 
    {
	starting();
    }
    elsif ($mode eq "stop")
    {
	stoping();
    }
    else
    {
	$event->set_event("ERROR", time(), "ERROR : ".localtime(time())." : Mode non reconnu.");
	quit();
    }
}

## ###################################
## Quit

sub	quit {
    $log->set_event_msg($event, "WARN", "ERROR", "INFO");
    exit (0);
}

## ###################################
## Starting

sub	starting {
    
    print "\nSTARTING ".$ifup." : ".$iface."\n";

    # Test si le bonding est actif ds le fichier de configuration
    
    if ($conf->get_value_file("actived")) {
	
	my $mode_b;
	my $miimon;
	
	# Test de la presence de differentes options
	unless ($miimon = $conf->get_value_file("miimon")) {
	    $event->set_event("ERROR", time(), "ERROR : ".localtime(time())." : miimon non present.");
	    quit();
	}
	unless ($mode_b = $conf->get_value_file("mode")) {
	    $event->set_event("ERROR", time(), "ERROR : ".localtime(time())." : mode non present.");
	    quit();
	}
	
	# Recuperation des couples cle/valeur sous forme de hash
	my %hash	= $conf->get_all();
	my %bondlist	= %{$hash{"bondlist_mode"}};
	my $max_bonds	= 0;
	
	# Compte du nombre d'interface bond pour le charchement du module
	foreach my $keys (keys(%{$hash{"bondlist_mode"}})) {
	    $max_bonds++;
	}
	
	my $error	= "";
	my $line	= "";
	my $execute	= "";
	
	# Test du Chargement du module bonding
	eval {$execute = `lsmod | grep bonding`};
	if ($@) {
	    $event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors de lexecution de lsmod : ".$@);
	}
	
	# Si le bonding est deja charge
	if ($execute) { 
	    # recuperation de la ligne bonding ds le fichier modules
	    $line = `cat /etc/modules | grep bonding`;
	    
	    # Split de cette ligne afin de recuperer ses arguments
	    my @result = split(" ", $line);
	    
	    ## Parsing des options de bonding dans /etc/modules
	    ## modification du fichier de conf en fonction
	    foreach (@result)
	    {
		if ($_ =~ m/^(.*)=(.*)$/) {
		    unless ($conf->set_value_file($1, $2)){
			$event->set_event("WARN", time(), "WARN : ".localtime(time())." : impossible de modifier le couple cle/valeur : ".$_);
		    }
		}
	    }
	} ## Sinon on le charge
	else {
	    eval {$execute =  `echo "bonding mode=$mode_b miimon=$miimon max_bonds=$max_bonds" >> /etc/modules`};
	    if ($@) {
		$event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors du parametrage de /etc/modules : ".$@);
	    }
	    
	    eval {$error = `modprobe bonding mode=$mode_b miimon=$miimon max_bonds=$max_bonds`};
	    if ($@) {
		$event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors du chargement du bonding : ".$@);
	    }
	}
	if ($error) {
	    $event->set_event("WARN", time(), "WARN : ".localtime(time())." : ".$error);
	}
	
	my $bond_name	= $iface;
	my @bond_list;

	# Si un nom d'interface a ete passe en parametre
	if ($bond_name)
	{
	    $bond_list[0] = $bond_name;
	}
	else # Sinon je recupere la liste des interfaces ds ifconfig
	{
	    eval {@bond_list = `ifconfig -a | cut -d " " -f1 | grep bond`};
	    if ($@) {
		$event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors de lexecution de ifconfig : ".$@);
	    }

	    chomp(@bond_list);
	}
	
	# Je parcours cette liste d'interface
	foreach $bond_name (@bond_list)
	{
	    
	    unless(defined($bondlist{$bond_name}))
	    {
		$conf->set_new_value_file($bond_name, ",", "bondlist_mode");
	    }
	    
	    if ($ifup && $ifup eq "pre" )
	    {
		# Je up les interfaces filles de l'interface bond
	    	my @fils = split(",", $bondlist{$bond_name});
		foreach (@fils)
		{
		    eval {`ifconfig $_ up`};
		    if ($@) {
			$event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors de lexecution de ifconfig $_ up: ".$@);
		    }
		}
		
		eval {`ifconfig $bond_name up`};
		if ($@) {
		    $event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors de lexecution de ifconfig $bond_name up: ".$@);
		}
	    }
	    
	    # Separation de la liste des interfaces fille par des espaces
	    $bondlist{$bond_name} =~ s/,/ /g;
	    
	    ## On fait le ifenslave
	    if ($ifup && $ifup eq "post") {
		if (-e "/sbin/ifenslave-2.6") {
		    if ($error = `ifenslave $bond_name $bondlist{$bond_name}`) {
			$event->set_event("WARN", time(), "WARN : ".localtime(time())." : ifenslave : ".$error);
		    }
		}
		else {
		    $event->set_event("ERROR", time(), "ERROR : ".localtime(time())." : ifenslave-2.6 non present");
		    quit();
		}
	    }
	}
    }
    else # Si le bonding est inactif dans le fichier de configuration
    {
	$event->set_event("INFO", time(), "INFO : ".localtime(time())." : Bonding non actif.");
    }
}

## ###################################
## Stoping

sub	stoping {

    print "\nSTOPING ".$ifup." : ".$iface."\n";

    my $line; 
    
    # on parcours la liste des bond a stoper dans le ifconfig
    my @tablist = `ifconfig | cut -d " " -f1 | grep bond`;
    chomp (@tablist);

    # On recupere les parametres dans le fichier de configuration
    # sous forme de hash
    my %hash = $conf->get_all();
    my %bondlist = %{$hash{"DEFAUTINTELLIQUEUNIQUE"}};

    # On parcours la liste des interfaces a stopper
    foreach my $bond_name (@tablist) 
    {
	
	eval {`ifconfig $bond_name down`};
	if ($@) {
	    $event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors de lexecution de ifconfig $bond_name down: ".$@);
	}

	my @interfaces;

	eval { @interfaces = `cat /etc/network/interfaces`};
	if ($@) {
	    $event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors de la lecture du fichier /etc/network/interfaces: ".$@);
	}
	
	# Ouverture du fichier interface pour supprimer
	# l'interface bond
	open (IFACE, "> /etc/network/interfaces");
	
	my $cpt = 0;
	
	foreach (@interfaces){
	    
	    if ($_ =~ m/^auto/)
	    {
		if ($bondlist{actived} == 0){
		    $_ =~ s/$bond_name/ /;
		    print IFACE $_;
		}
		else
		{
		    if ($_ !~ m/$bond_name/)
		    {
			chomp($_);
			print IFACE $_." $bond_name\n";
		    }
		    else {
			print IFACE $_;
		    }
		}
	    }
	    else {
		print IFACE $_;
	    } 
	    
	}
	close (IFACE);
	
    }
    
    ## Suppression du module bonding
    eval {`rmmod bonding`};
    if ($@) {
	$event->set_event("WARN", time(), "WARN : ".localtime(time())." : Probleme lors de rmmod bonding : ".$@);
    }
    
    # Pause de 2 secondes afin de laisser le temps au driver de stopper
    # le bonding
    sleep(2);


    # Suppression du bonding dans le fichier /etc/modules
    my @list = `cat /etc/modules`;
    open (MODULES, "> /etc/modules");
    foreach (@list) {
	unless ($_ =~ m/bonding/) {
	    print MODULES $_;
	}
    }
    close (MODULES);
}

## ###################################
## Main

sub	main {
    ## Recuperation des arguments, sinon affichage de l'aide 
    if (scalar(@ARGV)) {
	getCLOptions();
    } else {
	print_help();
	exit 1;
    }
    
    if ($help) {
	print_help();
	exit 1;
    }

    $debug = "DEBUG" if ( $debug );
    
    ## Creation des objets
    
    #Objet Event
    unless ($event = new Objet_Event($debug)) 
    {
	print "Pb creation Objet_Event\n";
	exit 1;
    }
    #Objet Conf
    unless ($conf = new Objet_Conf($event, $path_conf, "=", "#", $debug)) 
    {
	print "Pb creation Objet_Conf\n";
	exit 1;
    }
    # Recuperation de $path_log
    $conf->init();
    if ($conf->get_value_file("path_log")) {
	$path_log = $conf->get_value_file("path_log");
    }
    #Objet Log
    unless ($log = new Objet_Log($path_log, $debug))
    {
	print "Pb creation Objet_Log\n";
	exit 1;
    }
    if ($state)
    {
	print $conf->get_value_file("actived");
	quit();
    }
    
    unless ( $mode ) {
	$event->set_event("WARN", time(), "Warning : ".localtime(time())." : Aucun mode selectionne.");
    }
    
    # Si l'interface specifiee est de type bond
    # Si aucune interface n'est specifiee.

    if ($iface =~ m/bond/ || $iface eq "")
    {
	parse_mode($mode);
    }
    quit();
}

## ###################################
## Appel du main

main();
